//--------------------------------------------------------------
// File     : stm32_ub_scope_screen.h
//--------------------------------------------------------------

//--------------------------------------------------------------
#ifndef __STM32F4_UB_SCOPE_SCREEN_H
#define __STM32F4_UB_SCOPE_SCREEN_H


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32f4xx.h"
#include "stm32_ub_dac.h"
#include "stm32f4xx_rcc.h"
#include "stm32f4xx_tim.h"
#include "misc.h"



//--------------------------------------------------------------
// Array vom Scope-RAM
// (Max-Size = 256 x 256 Pixel !!)
//
// Werte muessen durch 8 Teilbar sein !!
// (bei UB ab 200Pixel keine Einzelpixel mehr zu erkennen)
//--------------------------------------------------------------
#define   SCOPE_SCREEN_MAXX   200  // Pixel in X-Richtung (max 256)
#define   SCOPE_SCREEN_MAXY   200  // Pixel in Y-Richtung (max 256)

#define   SCOPE_SCREEN_PIXEL     (SCOPE_SCREEN_MAXX*SCOPE_SCREEN_MAXY)  // gesammt Pixel
#define   SCOPE_SCREEN_RAM_LEN   (SCOPE_SCREEN_PIXEL/8)                 // laenge vom Array
#define   SCOPE_SCREEN_RAM_XLEN  (SCOPE_SCREEN_MAXX/8)                  // laenge einer X-Zeile

uint8_t   SCOPE_SCREEN_RAM[SCOPE_SCREEN_RAM_LEN];                       // 8pixel pro Eintrag


//--------------------------------------------------------------
#if SCOPE_SCREEN_MAXX > 256 || SCOPE_SCREEN_MAXY > 256
#error Max Pixel = 256
#endif


//--------------------------------------------------------------
// Faktor fuer die Anzeige
// zum ausrechnen der DAC-Werte
//--------------------------------------------------------------
#define   SCOPE_DAC_VALUE     4096  // 0 bis 4095
#define   SCOPE_FAKTOR_X    (SCOPE_DAC_VALUE/SCOPE_SCREEN_MAXX)
#define   SCOPE_FAKTOR_Y    (SCOPE_DAC_VALUE/SCOPE_SCREEN_MAXY)




//--------------------------------------------------------------
// Struktur von einem Pixel
// (X- und Y- Position)
//--------------------------------------------------------------
typedef struct {
  uint8_t xpos;   // X-Position
  uint8_t ypos;   // Y-Position
}Scope_Pixel_t;



//--------------------------------------------------------------
// defines vom Timer (TIM7)
// F_TIM = 84 MHz / (prescaler+1) / (periode+1)
// F=200kHz
// (damit koennen 2000 Punkte mit 100Hz dargestellt werden)
//--------------------------------------------------------------
#define   SCOPE_TIMER          TIM7
#define   SCOPE_TIMER_CLK      RCC_APB1Periph_TIM7
#define   SCOPE_TIMER_IRQ      TIM7_IRQn
#define   SCOPE_TIMER_ISR      TIM7_IRQHandler
#define   SCOPE_TIM_PRESCALE   20
#define   SCOPE_TIM_PERIODE    19



//--------------------------------------------------------------
// Array der Pixel-Liste
// ethllt alle Pixel die gesetzt sind
// (bei mehr als 3000 Pixel sinkt die Refreshrate unter 70 Hz)
//--------------------------------------------------------------
#define    SCOPE_MAX_PIXEL   3000  // max Pixel die gesetzt sein drfen
Scope_Pixel_t SCOPE_PIXEL_LIST[SCOPE_MAX_PIXEL];


//--------------------------------------------------------------
// Struktur von einem Font (max 16 Pixel Breite)
//--------------------------------------------------------------
typedef struct UB_Font_t {
  const uint16_t *table; // Tabelle mit den Daten
  uint16_t width;        // Breite eines Zeichens (in Pixel)
  uint16_t height;       // Hoehe eines Zeichens  (in Pixel)
}UB_Font;



//--------------------------------------------------------------
// Aktivierung der verschiedenen Fonts
// (unbenutzte einfach auskommentieren)
// fuer alle benutzten Fonts muss das entsprechende C-File
// in der CooCox-IDE hinzugefgt werden
//--------------------------------------------------------------
extern UB_Font Arial_7x10;
extern UB_Font Arial_8x13;
extern UB_Font Arial_10x15;
extern UB_Font Arial_11x18;
extern UB_Font Arial_13x19;
extern UB_Font Arial_14x22;
extern UB_Font Arial_16x25;



//--------------------------------------------------------------
// Struktur vom ScopeScreen
//--------------------------------------------------------------
typedef struct {
  uint32_t pixel_anz;      // anzahl der Pixel die gesetzt sind
  uint32_t akt_pixel_anz;  // aktuelle pixel anzahl
  uint32_t akt_list_pos;   // aktuelle listen position die angezeigt wird
}ScopeScreen_t;
ScopeScreen_t ScopeScreen;


//--------------------------------------------------------------
// Globale Funktionen
//--------------------------------------------------------------
void UB_ScopeScreen_Init(void);
void UB_ScopeScreen_Clear(void);
void UB_ScopeScreen_SetPixel(uint16_t xpos, uint16_t ypos);
void UB_ScopeScreen_ClearPixel(uint16_t xpos, uint16_t ypos);
void UB_ScopeScreen_DrawLine(int16_t x1, int16_t y1, int16_t x2, int16_t y2);
void UB_ScopeScreen_DrawCircle(int16_t x0, int16_t y0, int16_t radius);
void UB_ScopeScreen_DrawChar(uint16_t x, uint16_t y, uint8_t ascii, UB_Font *font);
void UB_ScopeScreen_DrawString(uint16_t x, uint16_t y,char *ptr, UB_Font *font);
ErrorStatus UB_ScopeScreen_RedrawScreen(void);
void UB_ScopeScreen_Testpattern(void);


//--------------------------------------------------------------
#endif // __STM32F4_UB_SCOPE_SCREEN_H
